@@ -5,10 +5,10 @@ import logging.handlers
 from webassets.filter import Filter
 
 
-__all__ = ('CSSUtilsFilter',)
+__all__ = ('CSSUtils',)
 
 
-class CSSUtilsFilter(Filter):
+class CSSUtils(Filter):
     """Minifies CSS by removing whitespace, comments etc., using the Python
     `cssutils <http://cthedot.de/cssutils/>`_ library.
 
@@ -17,24 +17,26 @@ class CSSUtilsFilter(Filter):
     """
 
     name = 'cssutils'
+    max_debug_level = None
 
     def setup(self):
         import cssutils
         self.cssutils = cssutils
 
         try:
-            # cssutils logs to stdout by default, hide that in production
-            if not self.env.debug:
-                log = logging.getLogger('assets.cssutils')
-                log.addHandler(logging.handlers.MemoryHandler(10))
+            # cssutils is unaware of so many new CSS3 properties,
+            # vendor-prefixes etc., that it's diagnostic messages are rather
+            # useless. Disable them.
+            log = logging.getLogger('assets.cssutils')
+            log.addHandler(logging.handlers.MemoryHandler(10))
 
-                # Newer versions of cssutils print a deprecation warning
-                # for 'setlog'.
-                if hasattr(cssutils.log, 'setLog'):
-                    func = cssutils.log.setLog
-                else:
-                    func = cssutils.log.setlog
-                func(log)
+            # Newer versions of cssutils print a deprecation warning
+            # for 'setlog'.
+            if hasattr(cssutils.log, 'setLog'):
+                func = cssutils.log.setLog
+            else:
+                func = cssutils.log.setlog
+            func(log)
         except ImportError:
             # During doc generation, Django is not going to be setup and will
             # fail when the settings object is accessed. That's ok though.
