@@ -2,10 +2,10 @@ from jspacker import JavaScriptPacker
 from webassets.filter import Filter
 
 
-__all__ = ('JSPackerFilter',)
+__all__ = ('JSPacker',)
 
 
-class JSPackerFilter(Filter):
+class JSPacker(Filter):
     """Reduces the size of Javascript using an inline compression
     algorithm, i.e. the script will be unpacked on the client side
     by the browser.
@@ -21,4 +21,4 @@ class JSPackerFilter(Filter):
         out.write(JavaScriptPacker().pack(_in.read(),
                                           compaction=False,
                                           encoding=62,
-                                          fastDecode=True))
\ No newline at end of file
+                                          fastDecode=True))
