@@ -6,13 +6,21 @@ I wonder whether I shouldn't just do full-stack tests here instead of mocking.
 
 from __future__ import with_statement
 
-from os import path
 import logging
 from threading import Thread, Event
 from nose.tools import assert_raises
+from nose import SkipTest
 import time
+
+try:
+    import argparse
+except ImportError:
+    raise SkipTest()
+
 from webassets import Bundle
-from webassets.script import main, CommandLineEnvironment, CommandError
+from webassets.exceptions import BuildError
+from webassets.script import (
+    main, CommandLineEnvironment, CommandError, GenericArgparseImplementation)
 from webassets.test import TempEnvironmentHelper
 from webassets.utils import working_directory
 
@@ -141,15 +149,26 @@ class TestBuildCommand(TestCLI):
         self.cmd_env.build(manifest='file:miau')
         assert self.exists('media/sub/miau')
 
+    def test_build_failure(self):
+        """If one bundle fails to build, the command continues, but
+        returns an error code in the end."""
+        def failing_filter(*a, **kw):
+            raise BuildError()
+        self.create_files(['file'])
+        a = Bundle('file', filters=failing_filter, output='outA')
+        self.assets_env.register('a', a)
+        b = Bundle('file', output='outB')
+        self.assets_env.register('b', b)
 
-class TestWatchCommand(TestCLI):
-    """This is a hard one to test.
+        # build() returns an error code
+        assert self.cmd_env.build() == 2
+        # the second bundle will have been built, event though the first failed
+        assert self.exists('outB')
+        assert not self.exists('outA')
 
-    We run the watch command in a thread, and rely on it's ``loop`` argument
-    to stop the thread again.
-    """
 
-    default_files = {'in': 'foo', 'out': 'bar'}
+class TestWatchMixin(object):
+    """Testing the watch command is hard."""
 
     def watch_loop(self):
         # Hooked into the loop of the ``watch`` command.
@@ -164,7 +183,7 @@ class TestWatchCommand(TestCLI):
         self.has_looped = Event()
         t = Thread(target=self.cmd_env.watch, kwargs={'loop': self.watch_loop})
         t.daemon = True   # In case something goes wrong with stopping, this
-                          # will allow the test process to be end nonetheless.
+        # will allow the test process to be end nonetheless.
         t.start()
         self.t = t
         # Wait for first iteration, which will initialize the mtimes. Only
@@ -173,6 +192,7 @@ class TestWatchCommand(TestCLI):
 
     def stop_watching(self):
         """Stop the watch command thread."""
+        assert self.t.isAlive() # If it has already ended, something is wrong
         self.stopped = True
         self.t.join(1)
 
@@ -182,6 +202,24 @@ class TestWatchCommand(TestCLI):
     def __exit__(self, exc_type, exc_val, exc_tb):
         self.stop_watching()
 
+
+class TestWatchCommand(TestWatchMixin, TestCLI):
+    """This is a hard one to test.
+
+    We run the watch command in a thread, and rely on its ``loop`` argument
+    to stop the thread again.
+    """
+
+    default_files = {'in': 'foo', 'out': 'bar'}
+
+    def setup(self):
+        super(TestWatchCommand, self).setup()
+
+        # Pay particular attention that the watch command works with auto_build
+        # disabled (since normally this implies no use of the updater, but
+        # obviously the command cannot pay attention to that).
+        self.env.auto_build = False
+
     def test(self):
         # Register a bundle to watch
         bundle = self.mkbundle('in', output='out')
@@ -200,7 +238,6 @@ class TestWatchCommand(TestCLI):
         # output file has been updated.
         assert self.get('out') == 'foo'
 
-
     def test_same_file_multiple_bundles(self):
         """[Bug] Test watch command can deal with the same file being part
         of multiple bundles. This was not always the case (github-127).
@@ -225,3 +262,105 @@ class TestWatchCommand(TestCLI):
         # Both output files have been updated.
         assert self.get('out') == 'foo'
         assert self.get('out2') == 'foo'
+
+    def test_initial_build(self):
+        """The watch command also detects changes that were made while it was
+        not running, and applies those right away on start.
+        """
+        # Register a bundle to watch
+        bundle = self.mkbundle('in', output='out')
+        self.env.register('test', bundle)
+
+        # Mark the input file has changed before we even run the command.
+        now = self.setmtime('in')
+        self.setmtime('out', mtime=now-100)
+
+        # Assert initial state
+        assert self.get('out') == 'bar'
+
+        # Run the watch command for a while, but don't make any changes.
+        with self:
+            time.sleep(0.2)
+
+        # Output file has been updated, not due to a change detected by watch,
+        # but because watch recognized the initial requirement for a build.
+        assert self.get('out') == 'foo'
+
+
+class TestCleanCommand(TestCLI):
+
+    def test(self):
+        self.create_files(['in'])
+        self.env.cache = True
+
+        bundle = self.mkbundle('in', output='out')
+        self.env.add(bundle)
+
+        bundle.build()
+        assert self.exists('out')
+        assert self.exists('.webassets-cache')
+
+        self.cmd_env.clean()
+        assert not self.exists('out')
+        assert not self.exists('.webassets-cache')
+
+
+class TestArgparseImpl(TestWatchMixin, TempEnvironmentHelper):
+    """Test the argparse-based implementation of the CLI interface."""
+
+    def test_no_env(self):
+        """[Regression] If no env is hardcoded, nor one given via
+        the commandline, we fail with a clean error.
+        """
+        impl = GenericArgparseImplementation(env=None)
+        assert_raises(CommandError, impl.run_with_argv, ['build'])
+
+    def test_watch_config_file(self):
+        """The watch command has an eye on the config file. This is an
+        extension to the base watch command."""
+        try:
+            import yaml
+        except ImportError:
+            raise SkipTest()
+
+        self.cmd_env = CommandLineEnvironment(self.env, logging)
+        self.cmd_env.commands['watch'] = \
+            GenericArgparseImplementation.WatchCommand(
+                self.cmd_env, argparse.Namespace(config=self.path('config.yml')))
+
+        self.create_files({'in': 'foo'})
+        template = """
+directory: .
+bundles:
+  foo:
+    contents:
+        - in
+    output: %s
+"""
+        self.create_files({'config.yml': template % 'outA'})
+
+        with self:
+            time.sleep(0.1)
+            # Change the config file; this change is detected; we update
+            # the timestamp explicitly or we might not have enough precision
+            self.create_files({'config.yml': template % 'outB'})
+            self.setmtime('config.yml', mod=100)
+            time.sleep(0.2)
+
+        # The second output file has been built
+        assert self.get('outB') == 'foo'
+
+    def test_watch_with_fixed_env_and_no_config(self):
+        """[Regression[ The custom 'watch' command does not break if the
+        CLI is initialized via fixed environment, instead of reading one from
+        a configuration file.
+        """
+        self.cmd_env = CommandLineEnvironment(self.env, logging)
+        self.cmd_env.commands['watch'] = \
+            GenericArgparseImplementation.WatchCommand(
+                self.cmd_env, argparse.Namespace())
+        with self:
+            time.sleep(0.1)
+        # No errors occured
+
+
